"""Dismantle package manager system with support for extensions and plugins."""

# IDEAS: --- plugins ----------------------------
# IDEAS: Function hook to provide plugin (process content before / after)
# IDEAS: List plugins for a function

_plugins = {}

from .iplugin import IPlugin
from .plugin import plugin
from .plugins import Plugins
from .register import register


__all__ = [
    'IPlugin',
    'register',
    'plugin',
    'Plugins',
    '_plugins'
]



from ._formats import (
    DirectoryPackageFormat,
    PackageFormat,
    TarPackageFormat,
    TgzPackageFormat,
    ZipPackageFormat
)
from ._handlers import HttpPackageHandler, LocalPackageHandler, PackageHandler


__all__ = [
    'HttpPackageHandler',
    'PackageFormat',
    'PackageHandler',
    'DirectoryPackageFormat',
    'LocalPackageHandler',
    'ZipPackageFormat',
    'TarPackageFormat',
    'TgzPackageFormat'
]



import shutil
from pathlib import Path
import pytest


@pytest.fixture(scope='function')
def datadir(tmpdir, request):
    """Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = Path(request.module.__file__)
    test_dir = Path(filename.parent, 'data', filename.stem)

    if test_dir.is_dir():
        shutil.rmtree(str(tmpdir), ignore_errors=True)
        shutil.copytree(test_dir, str(tmpdir))

    return tmpdir


@pytest.fixture(scope="session")
def httpserver_listen_address():
    """Use port 9090 for testing."""
    return ("127.0.0.1", 9090)



"""Dismantle package manager system with support for extensions and plugins."""

# IDEAS: --- extension --------------------------
# IDEAS: Class Hook to provide extensions (add new abilities such as log types)
# IDEAS: activate, deactivate, active

__all__ = [
    'IExtension',
    'Extensions'
]

from .iextension import IExtension
from .extensions import Extensions  # noqa: I001 (needs to be after iextension)



import os
from pathlib import Path
import pytest
from dismantle.package import PackageFormat, ZipPackageFormat


def test_inherits() -> None:
    assert issubclass(ZipPackageFormat, PackageFormat) is True


def test_grasp_exists(datadir: Path) -> None:
    src = datadir.join('package.zip')
    assert ZipPackageFormat.grasps(src) is True


def test_grasp_file_url(datadir: Path) -> None:
    src = f'file://{datadir.join("package.zip")}'
    assert ZipPackageFormat.grasps(src) is True


def test_grasp_not_supported(datadir: Path) -> None:
    src = datadir.join('directory_src')
    assert ZipPackageFormat.grasps(src) is False


def test_grasp_not_supported_format(datadir: Path) -> None:
    src = datadir.join('invalid.file')
    assert ZipPackageFormat.grasps(src) is False


def test_extract_not_supported(datadir: Path) -> None:
    src = datadir.join('directory_src')
    dest = datadir.join(f'{src}_output')
    message = 'formatter only supports zip files'
    with pytest.raises(ValueError, match=message):
        ZipPackageFormat.extract(src, dest)


def test_extract_not_supported_format(datadir: Path) -> None:
    src = datadir.join('invalid.file')
    dest = datadir.join(f'{src}_output')
    message = 'formatter only supports zip files'
    with pytest.raises(ValueError, match=message):
        ZipPackageFormat.extract(src, dest)


def test_extract_non_existant(datadir: Path) -> None:
    src = datadir.join('non_existant.zip')
    dest = datadir.join(f'{src}_output')
    message = 'invalid zip file'
    with pytest.raises(ValueError, match=message):
        ZipPackageFormat.extract(src, dest)


def test_extract_already_exists(datadir: Path) -> None:
    src = datadir.join('package.zip')
    dest = datadir.join('directory_exists')
    assert ZipPackageFormat.extract(src, dest) is None


def test_extract_create(datadir: Path) -> None:
    src = datadir.join('package.zip')
    dest = datadir.join('directory_created')
    ZipPackageFormat.extract(src, dest)
    assert os.path.exists(dest) is True
    assert os.path.exists(dest / 'package.json') is True



class IExtension:
    """Extension interface to recognise what needs to be processed."""

    _name: str = "default"
    _category: str = "default"

    def __init__(self) -> None:
        self._active: bool = False

    def activate(self) -> bool:
        self._active = True
        return self._active

    def deactivate(self) -> bool:
        self._active = False
        return not self._active

    @property
    def name(self) -> str:
        return self._name

    @property
    def category(self) -> str:
        return self._category

    @property
    def active(self) -> bool:
        return self._active
